--
-- Oracle 12c for Developers - SQL Samples
--
--
set echo on
--
-- OFFSET rows (TOP-N queries)
--
-- Base query
--
-- 1st SELECT show all rows in native order
select ename,sal from emp order by sal desc;
--
-- 2nd SELECT uses ROWNUM, note first 5 rows (before sorting)
select ename,sal from emp
    where rownum < 5 order by sal desc;
--
-- Fetch top 5 rows NEW!
--
select ename,sal from emp
      order by sal desc
      fetch first 5 rows only;
--
-- OFFSET (Pagination)
--
select ename,sal from emp
   order by sal desc
   offset 2 rows  
   fetch first 5 rows only;
--
-- PERCENTAGE
--
select ename,sal from emp
   order by sal desc
   offset 2 rows  
   fetch first 5 percent rows only;
--
--
-- Pattern matching - MATCH_RECOGNIZE
--    -- PARTITION BY / ORDER BY / PATTERN
select * from sh.sales
 match_recognize(
   partition by prod_id
   order by time_id
   measures strt.time_id as start_date,
   last(down.time_id) as bottom_date,
   last(up.time_id) as   end_date
   one row per match
   after match skip to last up
   pattern (strt down+ up+)
   define
     down as down.amount_sold < prev(down.amount_sold),
     up as up.amount_sold > prev(up.amount_sold)
   ) matcher
where prod_id = 148
order by matcher.prod_id, matcher.start_date;
--
select * from (select prod_id,trunc(time_id) time_id,sum(amount_sold) amount_sold from sh.sales where prod_id = 148 and extract(year from time_id) in (2000,2001) group by prod_id, trunc(time_id))
 match_recognize (
   partition by prod_id
   order by time_id
   measures to_char(strt.time_id,'yyyy-mm-dd') as start_date,
   to_char(last(down.time_id),'yyyy-mm-dd') as bottom_date,
   to_char(last(up.time_id) ,'yyyy-mm-dd') as   end_date,
   last(round(down.amount_sold)) as bottom_amt,
   last(round(up.amount_sold)) as end_amt
   --one row per match
   after match skip to last up
   pattern (strt down+ up+)
   define
     down as down.amount_sold < prev(down.amount_sold),
     up as up.amount_sold > prev(up.amount_sold)
   ) matcher
order by matcher.prod_id, matcher.start_date;
--
--
-- New use of Oracle Outer Join operator
--
select region_name, country_name, department_name, city, count(employee_id) nbr_emps
  from hr.regions r, hr.countries c, hr.locations l, hr.departments d, hr.employees e
  where r.region_id = c.region_id(+)
    and c.country_id = l.country_id(+)
    and l.location_id = d.location_id(+)
    and d.department_id = e.department_id(+)
    group by region_name,country_name,department_name, city
    order by region_name,country_name,department_name, city;
--
--
-- Using JOIN with Collection object CROSS APPLY  OUTER APPLY
--
create or replace type name_table_type as table of varchar2(100);
/
create or replace function department_employees (in_department_id varchar2)
   return name_table_type
is
  mynames name_table_type;
begin
  select cast(collect(last_name || ', ' || first_name) 			as name_table_type)
     into mynames
     from hr.employees
     where department_id = in_department_id;
  return mynames;
end;
/
select *
  from hr.departments d
       cross apply
       department_employees(d.department_id) dept_emps;
select *
  from hr.departments d
       outer apply
       department_employees(d.department_id) dept_emps;
select department_name
      ,department_employees(department_id) deptemps
  from hr.departments;
--
--
select prod_id,trunc(time_id) time_id,round(sum(amount_sold)) amount_sold from sh.sales where prod_id = 148 and extract(year from time_id) in (2000,2001) group by prod_id, trunc(time_id)
order by prod_id,trunc(time_id);
--
--
-- LATERAL inline views
--
select last_name,first_name,department_name
   from hr.employees e, (select * from hr.departments d where e.department_id = d.department_id);
--
select last_name,first_name,department_name
   from hr.employees e, LATERAL(select * from hr.departments d where e.department_id = d.department_id);
--
--
-- IDENTITY
--
drop table id_test1;
create table id_test1
(id number generated as identity,
 col1 varchar2(10));
--
insert into id_test1 (col1) values ('A');
insert into id_test1 (col1) values ('B');
insert into id_test1 (col1) values ('C');
--
select * from id_test1;

--
drop table id_test2;
create table id_test2
(id number generated as identity (	start with 10 increment by 11),
 col1 varchar2(10));
--
insert into id_test2 (col1) values ('A');
insert into id_test2 (col1) values ('B');
insert into id_test2 (col1) values ('C');
--
select * from id_test2; 
-- 
--
-- DEFAULTS
--
drop sequence default_test_seq;
drop table default_test;
create sequence default_test_seq start with 1 increment by 1;
create table default_test
(id number default default_test_seq.nextval not null,
 col1 varchar2(10) ,
 col2 varchar2(10)default on null 'N/A' not null);
insert into default_test (col1,col2) values ('A',null);
insert into default_test (col1) values ('B');
insert into default_test (col1,col2) values ('C','test');
select * from default_test;
--
--
-- INVISIBLE COLUMNS
drop table invisible_test;
create table invisible_test (
 id  number,
 col1 varchar2(10),
 col2 varchar2(10) invisible,
 col3 varchar2(10));
desc invisible_test;
insert into invisible_test
(col1,col2,col3) values (1,'a','a');
insert into invisible_test
(col1,col3) values (2,'b');
-- insert into invisible_test values (3,'c');  -- stmt FAILS, all VISIBLE columns must have values
insert into invisible_test values (3,'c','c');
select * from invisible_test;
alter table invisible_test modify col2 visible;
desc invisible_test;
--
--
-- Using Function in WITH (SH.SALES  918843 rows)
--
-- 1.33 first .66 second .65 third
-- 1.29 first 1.30 second 1.32 third
-- 3.49 first 3.63 second 3.39 third 
--
with function times_42(inval number)
return number
as
begin
 return inval * 42;
end;
select ename,empno,times_42(empno) from scott.emp;
--
--
create or replace function times_42(inval number)
 return number
as
begin
  return inval * 42;
end;
/
--
--
select channel_id,count(*) nbr_rows,sum(quantity_sold) qtysold,sum(times_42(cust_id)) cust42
  from sh.sales
  group by channel_id
  order by channel_id;
--
--
create or replace function times_42(inval number)
 return number
 result_cache
as
begin
  return inval * 42;
end;
/
--
select channel_id,count(*) nbr_rows,sum(quantity_sold) qtysold,sum(times_42(cust_id)) cust42
  from sh.sales
  group by channel_id
  order by channel_id;
--
--
--
-- TEMPORAL VALIDITY
--
--
drop table temporal_emp_test;
--
CREATE TABLE temporal_emp_test(
  employee_id NUMBER,
  last_name   VARCHAR2(50),
  start_time  TIMESTAMP,
  end_time    TIMESTAMP,
PERIOD FOR my_time_period (start_time, end_time));
 
INSERT INTO temporal_emp_test VALUES (1000, 'King', '01-Jan-10', '30-Jun-11');
INSERT INTO temporal_emp_test VALUES (1001, 'Manzo', '01-Jan-11', '30-Jun-11');
INSERT INTO temporal_emp_test VALUES (1002, 'Li', '01-Jan-12', null);
INSERT INTO temporal_emp_test VALUES (1003, 'Smith', '01-Jan-10', '31-Dec-10');
INSERT INTO temporal_emp_test VALUES (1005, 'Jones', '02-Jan-10', '31-Dec-10');
INSERT INTO temporal_emp_test VALUES (1006, 'Zelchik', '03-Jan-10', '30-Jun-11');
 
SELECT * from temporal_emp_test AS OF PERIOD FOR my_time_period TO_TIMESTAMP('01-Jun-10');

SELECT * from temporal_emp_test VERSIONS PERIOD FOR my_time_period BETWEEN 
   TO_TIMESTAMP('01-Jun-10') AND TO_TIMESTAMP('02-Jun-10');
--
commit;
--


